local function r96_luigi_voice(m)
    return m.character.type == CT_LUIGI
end

local LUIGI_VOICETABLE = {
    [CHAR_SOUND_ATTACKED] = 'luigi_attacked.mp3',
    [CHAR_SOUND_COUGHING1] = 'luigi_coughing_1.mp3',
    [CHAR_SOUND_COUGHING2] = 'luigi_coughing_1.mp3',
    [CHAR_SOUND_COUGHING3] = 'luigi_coughing_1.mp3',
    [CHAR_SOUND_DOH] = 'luigi_doh.mp3',
    [CHAR_SOUND_DROWNING] = 'luigi_drowning.mp3',
    [CHAR_SOUND_DYING] = 'luigi_dying.mp3',
    [CHAR_SOUND_EEUH] = 'luigi_pull_up.mp3',
    [CHAR_SOUND_GROUND_POUND_WAH] = 'luigi_yah.mp3',
    [CHAR_SOUND_HAHA] = 'luigi_haha.mp3',
    [CHAR_SOUND_HAHA_2] = 'luigi_haha.mp3',
    [CHAR_SOUND_HERE_WE_GO] = 'luigi_here_we_go.mp3',
    [CHAR_SOUND_HOOHOO] = {'luigi_hoohoo.mp3', 'luigi_hoohoo.mp3'},
    [CHAR_SOUND_HRMM] = 'luigi_lift.mp3',
    [CHAR_SOUND_IMA_TIRED] = 'luigi_yawning.mp3',
    [CHAR_SOUND_LETS_A_GO] = 'luigi_lets_a_go.mp3',
    [CHAR_SOUND_MAMA_MIA] = 'luigi_mama_mia.mp3',
    [CHAR_SOUND_OKEY_DOKEY] = 'luigi_okey_dokey.mp3',
    [CHAR_SOUND_ON_FIRE] = 'luigi_on_fire.mp3',
    [CHAR_SOUND_OOOF] = 'luigi_ooof.mp3',
    [CHAR_SOUND_OOOF2] = 'luigi_ooof.mp3',
    [CHAR_SOUND_PANTING] = 'luigi_panting.mp3',
    [CHAR_SOUND_PANTING_COLD] = 'luigi_panting_cold.mp3',
    [CHAR_SOUND_PUNCH_HOO] = 'luigi_punch_hoo.mp3',
    [CHAR_SOUND_PUNCH_WAH] = 'luigi_wah.mp3',
    [CHAR_SOUND_PUNCH_YAH] = 'luigi_punch_yah.mp3',
    [CHAR_SOUND_SNORING1] = 'luigi_snoring_1.mp3',
    [CHAR_SOUND_SNORING2] = 'luigi_snoring_2.mp3',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'luigi_so_longa_bowser.mp3',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'luigi_twirl_bounce.mp3',
    [CHAR_SOUND_UH] = 'luigi_grunt.mp3',
    [CHAR_SOUND_UH2] = 'luigi_grunt_2.mp3',
    [CHAR_SOUND_UH2_2] = 'luigi_grunt_2.mp3',
    [CHAR_SOUND_WAAAOOOW] = 'luigi_falling.mp3',
    [CHAR_SOUND_WAH2] = 'luigi_wah.mp3',
    [CHAR_SOUND_WHOA] = 'luigi_whoa.mp3',
    [CHAR_SOUND_YAHOO] = 'luigi_yahoo.mp3',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'luigi_yahoo.mp3', 'luigi_yippee.mp3'},
    [CHAR_SOUND_YAH_WAH_HOO] = {'luigi_punch_yah.mp3', 'luigi_yah.mp3'},
}

gLuigiSamples = {}
gLuigiStream = nil

function stop_r96_luigi_sound(m, sound)
    local voice_sample = gLuigiSamples[m.playerIndex]
    if voice_sample == nil or not voice_sample.loaded then
        return
    end

    audio_sample_stop(voice_sample)
    if voice_sample.file.relativePath:match('^.+/(.+)$') == sound then
        return voice_sample
    end
    audio_sample_destroy(voice_sample)
end

function play_r96_luigi_sound(m, voice)
    --Get sound, if it's a table, get a random entry from it
    local sound
    if type(voice) == "table" then
        sound = voice[math.random(#voice)]
    else
        sound = voice
    end
    if sound == nil then return 0 end

    --Get current sample and stop it
    local voice_sample = stop_r96_luigi_sound(m, sound)

    --If the new sound isn't a string, let's assume it's
    --a number to return to the character sound hook
    if type(sound) ~= "string" then
        return sound
    end

    --Load a new sample and play it! Don't make a new one if we don't need to
    if (m.area == nil or m.area.camera == nil) and m.playerIndex == 0 then
        if gLuigiStream ~= nil then
            audio_stream_stop(gLuigiStream)
            audio_stream_destroy(gLuigiStream)
        end
        gLuigiStream = audio_stream_load(sound)
        audio_stream_play(gLuigiStream, true, 1)
    else
        if voice_sample == nil then
            voice_sample = audio_sample_load(sound)
        end
        audio_sample_play(voice_sample, m.pos, 1)

        gLuigiSamples[m.playerIndex] = voice_sample
    end
    return 0
end

local function r96_luigi_sound(m, characterSound)
    if not r96_luigi_voice(m) then return end
    if characterSound == CHAR_SOUND_SNORING3 then return 0 end
    if characterSound == CHAR_SOUND_HAHA and m.hurtCounter > 0 then return 0 end

    local voice = LUIGI_VOICETABLE[characterSound]
    if voice ~= nil then
        return play_r96_luigi_sound(m, voice)
    end
    return 0
end

hook_event(HOOK_CHARACTER_SOUND, r96_luigi_sound)