local function r96_mario_voice(m)
    return m.character.type == CT_MARIO
end

local MARIO_VOICETABLE = {
    [CHAR_SOUND_ATTACKED] = 'mario_attacked.mp3',
    [CHAR_SOUND_COUGHING1] = 'mario_coughing_1.mp3',
    [CHAR_SOUND_COUGHING2] = 'mario_coughing_1.mp3',
    [CHAR_SOUND_COUGHING3] = 'mario_coughing_1.mp3',
    [CHAR_SOUND_DOH] = 'mario_doh.mp3',
    [CHAR_SOUND_DROWNING] = 'mario_drowning.mp3',
    [CHAR_SOUND_DYING] = 'mario_dying.mp3',
    [CHAR_SOUND_EEUH] = 'mario_pull_up.mp3',
    [CHAR_SOUND_GROUND_POUND_WAH] = 'mario_yah.mp3',
    [CHAR_SOUND_HAHA] = 'mario_haha.mp3',
    [CHAR_SOUND_HAHA_2] = 'mario_haha.mp3',
    [CHAR_SOUND_HERE_WE_GO] = 'mario_here_we_go.mp3',
    [CHAR_SOUND_HOOHOO] = {'mario_hoohoo.mp3', 'mario_hoohoo.mp3'},
    [CHAR_SOUND_HRMM] = 'mario_lift.mp3',
    [CHAR_SOUND_IMA_TIRED] = 'mario_yawning.mp3',
    [CHAR_SOUND_LETS_A_GO] = 'mario_lets_a_go.mp3',
    [CHAR_SOUND_MAMA_MIA] = 'mario_mama_mia.mp3',
    [CHAR_SOUND_OKEY_DOKEY] = 'mario_okey_dokey.mp3',
    [CHAR_SOUND_ON_FIRE] = 'mario_on_fire.mp3',
    [CHAR_SOUND_OOOF] = 'mario_ooof.mp3',
    [CHAR_SOUND_OOOF2] = 'mario_ooof.mp3',
    [CHAR_SOUND_PANTING] = 'mario_panting.mp3',
    [CHAR_SOUND_PANTING_COLD] = 'mario_panting_cold.mp3',
    [CHAR_SOUND_PUNCH_HOO] = 'mario_punch_hoo.mp3',
    [CHAR_SOUND_PUNCH_WAH] = 'mario_wah.mp3',
    [CHAR_SOUND_PUNCH_YAH] = 'mario_punch_yah.mp3',
    [CHAR_SOUND_SNORING1] = 'mario_snoring_1.mp3',
    [CHAR_SOUND_SNORING2] = 'mario_snoring_2.mp3',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'mario_so_longa_bowser.mp3',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'mario_twirl_bounce.mp3',
    [CHAR_SOUND_UH] = 'mario_grunt.mp3',
    [CHAR_SOUND_UH2] = 'mario_grunt_2.mp3',
    [CHAR_SOUND_UH2_2] = 'mario_grunt_2.mp3',
    [CHAR_SOUND_WAAAOOOW] = 'mario_falling.mp3',
    [CHAR_SOUND_WAH2] = 'mario_wah.mp3',
    [CHAR_SOUND_WHOA] = 'mario_whoa.mp3',
    [CHAR_SOUND_YAHOO] = 'mario_yahoo.mp3',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'mario_yahoo.mp3', 'mario_yippee.mp3'},
    [CHAR_SOUND_YAH_WAH_HOO] = {'mario_punch_yah.mp3', 'mario_yah.mp3'},
}

gMarioSamples = {}
gMarioStream = nil

function stop_r96_mario_sound(m, sound)
    local voice_sample = gMarioSamples[m.playerIndex]
    if voice_sample == nil or not voice_sample.loaded then
        return
    end

    audio_sample_stop(voice_sample)
    if voice_sample.file.relativePath:match('^.+/(.+)$') == sound then
        return voice_sample
    end
    audio_sample_destroy(voice_sample)
end

function play_r96_mario_sound(m, voice)
    --Get sound, if it's a table, get a random entry from it
    local sound
    if type(voice) == "table" then
        sound = voice[math.random(#voice)]
    else
        sound = voice
    end
    if sound == nil then return 0 end

    --Get current sample and stop it
    local voice_sample = stop_r96_mario_sound(m, sound)

    --If the new sound isn't a string, let's assume it's
    --a number to return to the character sound hook
    if type(sound) ~= "string" then
        return sound
    end

    --Load a new sample and play it! Don't make a new one if we don't need to
    if (m.area == nil or m.area.camera == nil) and m.playerIndex == 0 then
        if gMarioStream ~= nil then
            audio_stream_stop(gMarioStream)
            audio_stream_destroy(gMarioStream)
        end
        gMarioStream = audio_stream_load(sound)
        audio_stream_play(gMarioStream, true, 1)
    else
        if voice_sample == nil then
            voice_sample = audio_sample_load(sound)
        end
        audio_sample_play(voice_sample, m.pos, 1)

        gMarioSamples[m.playerIndex] = voice_sample
    end
    return 0
end

local function r96_mario_sound(m, characterSound)
    if not r96_mario_voice(m) then return end
    if characterSound == CHAR_SOUND_SNORING3 then return 0 end
    if characterSound == CHAR_SOUND_HAHA and m.hurtCounter > 0 then return 0 end

    local voice = MARIO_VOICETABLE[characterSound]
    if voice ~= nil then
        return play_r96_mario_sound(m, voice)
    end
    return 0
end

hook_event(HOOK_CHARACTER_SOUND, r96_mario_sound)