local function r96_wario_voice(m)
    return m.character.type == CT_WARIO
end

local WARIO_VOICETABLE = {
    [CHAR_SOUND_ATTACKED] = 'wario_attacked.mp3',
    [CHAR_SOUND_COUGHING1] = 'wario_coughing_1.mp3',
    [CHAR_SOUND_COUGHING2] = 'wario_coughing_1.mp3',
    [CHAR_SOUND_COUGHING3] = 'wario_coughing_1.mp3',
    [CHAR_SOUND_DOH] = 'wario_doh.mp3',
    [CHAR_SOUND_DROWNING] = 'wario_drowning.mp3',
    [CHAR_SOUND_DYING] = 'wario_dying.mp3',
    [CHAR_SOUND_EEUH] = 'wario_pull_up.mp3',
    [CHAR_SOUND_GROUND_POUND_WAH] = 'wario_yah.mp3',
    [CHAR_SOUND_HAHA] = 'wario_haha.mp3',
    [CHAR_SOUND_HAHA_2] = 'wario_haha.mp3',
    [CHAR_SOUND_HERE_WE_GO] = 'wario_here_we_go.mp3',
    [CHAR_SOUND_HOOHOO] = {'wario_hoohoo.mp3', 'wario_hoohoo.mp3'},
    [CHAR_SOUND_HRMM] = 'wario_lift.mp3',
    [CHAR_SOUND_IMA_TIRED] = 'wario_yawning.mp3',
    [CHAR_SOUND_LETS_A_GO] = 'wario_lets_a_go.mp3',
    [CHAR_SOUND_MAMA_MIA] = 'wario_mama_mia.mp3',
    [CHAR_SOUND_OKEY_DOKEY] = 'wario_okey_dokey.mp3',
    [CHAR_SOUND_ON_FIRE] = 'wario_on_fire.mp3',
    [CHAR_SOUND_OOOF] = 'wario_ooof.mp3',
    [CHAR_SOUND_OOOF2] = 'wario_ooof.mp3',
    [CHAR_SOUND_PANTING] = 'wario_panting.mp3',
    [CHAR_SOUND_PANTING_COLD] = 'wario_panting_cold.mp3',
    [CHAR_SOUND_PUNCH_HOO] = 'wario_punch_hoo.mp3',
    [CHAR_SOUND_PUNCH_WAH] = 'wario_wah.mp3',
    [CHAR_SOUND_PUNCH_YAH] = 'wario_punch_yah.mp3',
    [CHAR_SOUND_SNORING1] = 'wario_snoring_1.mp3',
    [CHAR_SOUND_SNORING2] = 'wario_snoring_2.mp3',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'wario_so_longa_bowser.mp3',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'wario_twirl_bounce.mp3',
    [CHAR_SOUND_UH] = 'wario_grunt.mp3',
    [CHAR_SOUND_UH2] = 'wario_grunt_2.mp3',
    [CHAR_SOUND_UH2_2] = 'wario_grunt_2.mp3',
    [CHAR_SOUND_WAAAOOOW] = 'wario_falling.mp3',
    [CHAR_SOUND_WAH2] = 'wario_wah.mp3',
    [CHAR_SOUND_WHOA] = 'wario_whoa.mp3',
    [CHAR_SOUND_YAHOO] = 'wario_yahoo.mp3',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'wario_yahoo.mp3', 'wario_yippee.mp3'},
    [CHAR_SOUND_YAH_WAH_HOO] = {'wario_punch_yah.mp3', 'wario_yah.mp3'},
}

gWarioSamples = {}
gWarioStream = nil

function stop_r96_wario_sound(m, sound)
    local voice_sample = gWarioSamples[m.playerIndex]
    if voice_sample == nil or not voice_sample.loaded then
        return
    end

    audio_sample_stop(voice_sample)
    if voice_sample.file.relativePath:match('^.+/(.+)$') == sound then
        return voice_sample
    end
    audio_sample_destroy(voice_sample)
end

function play_r96_wario_sound(m, voice)
    --Get sound, if it's a table, get a random entry from it
    local sound
    if type(voice) == "table" then
        sound = voice[math.random(#voice)]
    else
        sound = voice
    end
    if sound == nil then return 0 end

    --Get current sample and stop it
    local voice_sample = stop_r96_wario_sound(m, sound)

    --If the new sound isn't a string, let's assume it's
    --a number to return to the character sound hook
    if type(sound) ~= "string" then
        return sound
    end

    --Load a new sample and play it! Don't make a new one if we don't need to
    if (m.area == nil or m.area.camera == nil) and m.playerIndex == 0 then
        if gWarioStream ~= nil then
            audio_stream_stop(gWarioStream)
            audio_stream_destroy(gWarioStream)
        end
        gWarioStream = audio_stream_load(sound)
        audio_stream_play(gWarioStream, true, 1)
    else
        if voice_sample == nil then
            voice_sample = audio_sample_load(sound)
        end
        audio_sample_play(voice_sample, m.pos, 1)

        gWarioSamples[m.playerIndex] = voice_sample
    end
    return 0
end

local function r96_wario_sound(m, characterSound)
    if not r96_wario_voice(m) then return end
    if characterSound == CHAR_SOUND_SNORING3 then return 0 end
    if characterSound == CHAR_SOUND_HAHA and m.hurtCounter > 0 then return 0 end

    local voice = WARIO_VOICETABLE[characterSound]
    if voice ~= nil then
        return play_r96_wario_sound(m, voice)
    end
    return 0
end

hook_event(HOOK_CHARACTER_SOUND, r96_wario_sound)